<?php

use GuzzleHttp\Client;
use function Stringy\create as S;


class Metrics
{
    protected Client $client;

    public function __construct(string $url, string $user, string $password, string $database)
    {
        $this->client = new Client([
          'base_uri'         => $url,
          'auth'             => [$user, $password],
          'headers'          => [
            'Accept'          => 'application/json',
            'Accept-Charset'  => 'utf-8',
            'Accept-Language' => 'ru',
            'Accept-Encoding' => 'gzip',
            'Cache-Control'   => 'no-cache',
            'Pragma'          => 'no-cache',
            'User-Agent'      => NULL,
          ],
          'verify'           => true,
          'http_errors'      => false,
          'synchronous'      => true,
          'force_ip_resolve' => 'v4',
          'timeout'          => 10,
          'connect_timeout'  => 3,
          'read_timeout'     => 5,
          'allow_redirects'  => false,
          'delay'            => 250,
          'query'            => [
            'db'        => $database,
            'u'         => $user,
            'password'  => $password,
            'precision' => 's',
          ],
        ]);
    }

    public function send(string $measurement, array $fields, array $tags = [], int $timestamp = 0): void
    {
        $measurement = S($measurement)
          ->collapseWhitespace()
          ->replace(',', '\,')
          ->replace(' ', '\ ')
          ->toString();

        foreach($fields as $key => $value) {
            $key = S($key)
              ->collapseWhitespace()
              ->replace(',', '\,')
              ->replace(' ', '\ ')
              ->replace('=', '\=')
              ->toString();

            $value = S($value);
            if($value->containsAny(['.', ','])) {
                $value = $value->replace(',', '.')->toString();
            } else {
                $value = $value->append('.0')->toString();
            }

            $fields[$key] = $value;
        }

        foreach($tags as $key => $value) {
            $key = S($key)
              ->collapseWhitespace()
              ->replace(',', '\,')
              ->replace(' ', '\ ')
              ->replace('=', '\=')
              ->toString();

            $value = S($value)
              ->collapseWhitespace()
              ->replace(',', '\,')
              ->replace(' ', '\ ')
              ->replace('=', '\=')
              ->toString();

            $tags[$key] = $value;
        }

        $timestamp = $timestamp > 0 ? $timestamp : time();

        $body = $this->toString($measurement, $tags, $fields, $timestamp);

        $this->client->post('write', ['body' => $body]);
    }

    protected function toString(string $measurement, array $tags, array $fields, int $timestamp): string
    {
        $line = S($measurement);

        foreach($tags as $key => $value) {
            $line = $line->append(',')
              ->append($key)
              ->append('=')
              ->append($value);
        }

        $line = $line->append(' ');

        foreach($fields as $key => $value) {
            $line = $line->append($key)
              ->append('=')
              ->append($value)
              ->append(',');
        }

        $line = $line->removeRight(',')
          ->append(' ')
          ->append($timestamp);

        return $line->toString();
    }

}